package com.yovole.sdk.common.model;

import java.io.Serializable;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class Pagination<T> implements Serializable {
    private int curPage;
    private int perPage;
    private int totalRecords;
    private int totalPages;
    private List<T> pageData;
    private boolean autoCompute;
    private int pageOffset;
    private boolean more;
    private Map<String, String> queryConditions;
    private Map<String, String> orderBys;

    public Pagination() {
        this(1, 10);
    }

    public Pagination(int curPage, int perPage) {
        this(curPage, perPage, 0);
    }

    public Pagination(int curPage, int perPage, int totalRecords) {
        this(curPage, perPage, totalRecords, (List)null);
    }

    public Pagination(int curPage, int perPage, int totalRecords, List<T> pageData) {
        this.curPage = 1;
        this.perPage = 10;
        this.totalRecords = 0;
        this.totalPages = 0;
        this.pageData = null;
        this.autoCompute = true;
        this.more = false;
        this.queryConditions = new LinkedHashMap();
        this.orderBys = new LinkedHashMap();
        this.autoCompute = false;
        this.setCurPage(curPage);
        this.setPerPage(perPage);
        this.setTotalRecords(totalRecords);
        this.setPageData(pageData);
        this.autoCompute = true;
        this.compute();
    }

    public Pagination(int curPage, int perPage, List<T> pageData) {
        this.curPage = 1;
        this.perPage = 10;
        this.totalRecords = 0;
        this.totalPages = 0;
        this.pageData = null;
        this.autoCompute = true;
        this.more = false;
        this.queryConditions = new LinkedHashMap();
        this.orderBys = new LinkedHashMap();
        this.autoCompute = false;
        this.setCurPage(curPage);
        this.setPerPage(perPage);
        this.setTotalRecords(pageData.size());
        this.setPageData(pageData);
        this.autoCompute = true;
        this.compute();
    }

    public boolean isMore() {
        return this.more;
    }

    public void setMore(boolean more) {
        this.more = more;
    }

    private void compute() {
        if (this.autoCompute) {
            if (this.curPage < 1) {
                this.curPage = 1;
            }

            if (this.perPage <= 0) {
                this.totalPages = 1;
                this.pageOffset = 0;
                this.perPage = this.totalRecords;
            } else {
                this.totalPages = this.totalRecords / this.perPage;
                if (this.totalRecords % this.perPage != 0) {
                    ++this.totalPages;
                }

                if (this.curPage > this.totalPages && this.totalPages > 0) {
                    this.curPage = this.totalPages;
                }

                this.pageOffset = (this.curPage - 1) * this.perPage;
                if (this.pageOffset < 0) {
                    this.pageOffset = 0;
                }
            }
        }

    }

    public int getPageOffset() {
        return this.more ? 0 : this.pageOffset;
    }

    public int getTotalPages() {
        return this.totalPages;
    }

    public int getCurPage() {
        return this.curPage;
    }

    public void setCurPage(int curPage) {
        this.curPage = curPage;
        this.compute();
    }

    public int getPerPage() {
        return this.perPage;
    }

    public void setPerPage(int perPage) {
        this.perPage = perPage;
        this.compute();
    }

    public int getMaxResult() {
        if (this.more) {
            int max = this.pageOffset + this.perPage;
            return Math.min(max, this.totalRecords);
        } else {
            return this.perPage;
        }
    }

    public int getTotalRecords() {
        return this.totalRecords;
    }

    public void setTotalRecords(int totalRecords) {
        this.totalRecords = totalRecords;
        this.compute();
    }

    public List<T> getPageData() {
        return this.pageData;
    }

    public void setPageData(List<T> pageData) {
        this.pageData = pageData;
    }

    public Map<String, String> getQueryConditions() {
        return this.queryConditions;
    }

    public void setQueryConditions(Map<String, String> queryConditions) {
        this.queryConditions = queryConditions;
    }

    public void addQueryCondition(String fieldName, String value) {
        this.queryConditions.put(fieldName, value);
    }

    public Map<String, String> getOrderBys() {
        return this.orderBys;
    }

    public void setOrderBys(Map<String, String> orderBys) {
        this.orderBys = orderBys;
    }

    public void addOrderBy(String fieldName, String order) {
        this.orderBys.put(fieldName, order);
    }
}
