package com.yovole.sdk.support;

import com.yovole.sdk.common.common.Endpoint;
import com.yovole.sdk.common.common.EndpointType;
import com.yovole.sdk.common.common.Region;
import org.apache.log4j.Logger;
import org.json.JSONArray;
import org.json.JSONObject;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Author : dujie
 * Created : 2018/10/16 17:09
 */
public class Endpoints {
    private static List<Endpoint> endpoints = new ArrayList<>();
    private static Logger logger = Logger.getLogger(Endpoints.class);

    static {
        try {
            String contents = ClasspathResourceLoader.loadResource("endpoints.json");
            JSONArray ja = new JSONArray(contents);
            for(int i=0; i<ja.length(); i++) {
                JSONObject o = ja.getJSONObject(i);
                EndpointType type = EndpointType.valueOf(o.getString("type"));
                if(EndpointType.GLOBAL ==  type) {
                    endpoints.add(new Endpoint(type, o.getString("endpoint")));
                } else {
                    endpoints.add(new Endpoint(type, o.getString("endpoint"), o.getString("region")));
                }
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
    }

    public static List<Endpoint> getEndpoints() {
        return Collections.unmodifiableList(endpoints);
    }

    public static Endpoint getGlobalEndpoint() {
        for(Endpoint endpoint : endpoints) {
            if(EndpointType.GLOBAL == endpoint.getType()) return endpoint;
        }
        return null;
    }

    public static Endpoint getRegionEndpoint(String region) {
        for(Endpoint endpoint : endpoints) {
            if(EndpointType.REGION == endpoint.getType() && endpoint.getRegion().equals(region)) return endpoint;
        }
        return null;
    }

    public static Endpoint getRegionEndpoint(Region region) {
        return getRegionEndpoint(region.getCode());
    }
}