package com.yovole.sdk.project;

import com.yovole.sdk.common.auth.BasicCloudCredentials;
import com.yovole.sdk.common.auth.ICloudCredentials;
import com.yovole.sdk.common.common.Endpoint;
import com.yovole.sdk.common.common.EndpointType;
import com.yovole.sdk.common.utils.JsonUtils;
import com.yovole.sdk.project.request.*;
import com.yovole.sdk.project.response.ProjectCreateResponse;
import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;
import java.util.Properties;

/**
 * Author : dujie
 * Created : 2018/10/23 21:06
 */
public class ProjectApiSdkClientTest {
    private ProjectApiSdkClient client;
    private Logger logger = Logger.getLogger(getClass());
    @Before
    public void setUp() throws Exception {
        ICloudCredentials cloudCredentials = new BasicCloudCredentials("ed5e53be-7ff2-402f-82f5-0caaaba3754a", "ed4e2b3b-8f81-4b8e-a8f6-666583c5c579");
        client = new ProjectApiSdkClient(cloudCredentials, null);
        Endpoint endpoint = new Endpoint(EndpointType.GLOBAL, "api.yovole.com");
        endpoint.setProtocol("https");
        client.getProfile().setEndpoint(endpoint);
    }

    @After
    public void tearDown() throws Exception {

    }

    @Test
    public void createProject() {
        try {
            ProjectCreateRequest project = new ProjectCreateRequest();
            project.setName("API Create2");
            project.setColor("project-color-10");
            long start = System.currentTimeMillis();
            ProjectCreateResponse response = client.createProject(project);
            long end = System.currentTimeMillis();
            System.out.println(JsonUtils.toJson(response));
            System.out.println("call create project api using " + (end - start) + " ms");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void listProjects() throws Exception{
        ProjectListRequest request = new ProjectListRequest();
        request.setCurPage(1);
        logger.debug(client.listProjects(request));
    }


    @Test
    public void listProjectMembers() throws Exception{
        ProjectMemberListRequest request = new ProjectMemberListRequest();
        request.setSn("PRJ-4SMAN");
        logger.debug(client.listProjectMembers(request));
    }


    @Test
    public void addProjectMember() {
        ProjectMemberAddRequest request = new ProjectMemberAddRequest();
        request.setProjectSn("PRJ-4SMAN");
        request.setUserIds(new ArrayList<>());
        request.getUserIds().add("8a83a02861bcf9070161d188f6700207");
        request.setPermission(7);
        logger.debug(client.addProjectMember(request));
    }

    @Test
    public void removeProjectMember() {
        //			"id": "8a83a02b67621e1b0167733dc10d0362",
        //			"permission": 7,
        //			"projectId": "PRJ-4SMAN",
        //			"username": "linjing@yovole.com"
        ProjectMemberRemoveRequest removeRequest = new ProjectMemberRemoveRequest();
        removeRequest.setProjectSn("PRJ-4SMAN");
        removeRequest.setUserIds(new ArrayList<>());
        removeRequest.getUserIds().add("8a83a02b67621e1b0167733dc10d0362");
        logger.debug(client.removeProjectMember(removeRequest));
    }

    @Test
    public void updateProjectMember() {
        ProjectMemberUpdateRequest request = new ProjectMemberUpdateRequest();
        request.setProjectSn("PRJ-4SMAN");
        request.setUserIds(new ArrayList<>());
        request.getUserIds().add("8a83a02b67621e1b0167733dc10d0362");
        request.getUserIds().add("8a83a02861bcf9070161d188f6700207");
        request.setPermission(5);
        logger.debug(client.updateProjectMember(request));

    }
}