package com.yovole.sdk.oss.core;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.FileInputStream;
import java.io.InputStream;

public class ReleasableInputStream extends SdkFilterInputStream implements Releasable {
    private static final Log log = LogFactory
            .getLog(ReleasableInputStream.class);

    private boolean closeDisabled;

    protected ReleasableInputStream(InputStream is) {
        super(is);
    }

    @Override
    public final void close() {
        if (!closeDisabled)
            doRelease();
    }

    @Override
    public final void release() {
        doRelease();
    }

    private void doRelease() {
        try {
            in.close();
        } catch (Exception ex) {
            if (log.isDebugEnabled())
                log.debug("FYI", ex);
        }
        if (in instanceof Releasable) {

            Releasable r = (Releasable)in;
            r.release();
        }
        abortIfNeeded();
    }

    public final boolean isCloseDisabled() {
        return closeDisabled;
    }

    public final <T extends ReleasableInputStream> T disableClose() {
        this.closeDisabled = true;
        @SuppressWarnings("unchecked")
        T t = (T)this;
        return t;
    }

    public static ReleasableInputStream wrap(InputStream is) {
        if (is instanceof ReleasableInputStream)
            return (ReleasableInputStream)is;   // already wrapped
        if (is instanceof FileInputStream)
            return ResettableInputStream.newResettableInputStream((FileInputStream)is);
        return new ReleasableInputStream(is);
    }
}
