package com.yovole.sdk.oss.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.yovole.sdk.oss.utils.JacksonUtils;

import java.io.Serializable;

@JsonInclude(JsonInclude.Include.NON_NULL)
public abstract class BaseRequest implements Serializable,Cloneable {
    private static final long serialVersionUID = -7766291515849194580L;
    @JsonIgnore
    private String vppId;
    @JsonIgnore
    private String customerId;
    @JsonIgnore
    private String userId;
    @JsonIgnore
    private String bucket;
    @JsonIgnore
    private String key;
    @JsonIgnore
    private String subPath;

    protected static final String NOT_SPECIFIED_TEMP = "Required attribute '%s' not specified.";

    public abstract void validate();

    protected String getErrorMessage(String name) {
        return String.format(NOT_SPECIFIED_TEMP, name);
    }

    public <T extends BaseRequest> T withObject(String bucket,String key) {
        T t = (T) this;
        t.setBucket(bucket);
        t.setKey(key);
        return t;
    }

    public <T extends BaseRequest> T withUserInfo(String vppId, String customerId, String userId) {
        setCustomerId(customerId);
        setUserId(userId);
        setVppId(vppId);
        T t = (T) this;
        return t;
    }

    public <T extends BaseRequest> T withUserInfoAndObject(String vppId, String customerId, String userId,String bucket,String key) {
        T t = this.withUserInfo(vppId,customerId,userId);
        t.setBucket(bucket);
        t.setKey(key);
        return t;
    }

    @Override
    public String toString() {
        return JacksonUtils.objToString(this);
    }

    public String getBucket() {
        return bucket;
    }

    public void setBucket(String bucket) {
        this.bucket = bucket;
    }

    public String getKey() {
        return key;
    }

    public void setKey(String key) {
        this.key = key;
    }

    public String getSubPath() {
        return subPath;
    }

    public void setSubPath(String subPath) {
        this.subPath = subPath;
    }

    public String getVppId() {
        return vppId;
    }

    public void setVppId(String vppId) {
        this.vppId = vppId;
    }

    public String getCustomerId() {
        return customerId;
    }

    public void setCustomerId(String customerId) {
        this.customerId = customerId;
    }

    public String getUserId() {
        return userId;
    }

    public void setUserId(String userId) {
        this.userId = userId;
    }

    public <T extends BaseRequest> T withRequest(BaseRequest t1) {
        return this.withUserInfo(t1.getVppId(),t1.getCustomerId(),t1.getUserId());
    }

}
