package com.yovole.sdk.oss.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;


public class DeleteObjectsRequest extends BaseRequest {

    private static final long serialVersionUID = -3794525378326270007L;

    private final List<KeyVersion> keys = new ArrayList<KeyVersion>();

    public DeleteObjectsRequest(String bucketName) {
        setBucket(bucketName);
    }

    public void setKeys(List<KeyVersion> keys) {
        this.keys.clear();
        this.keys.addAll(keys);
    }

    public DeleteObjectsRequest withKeys(List<KeyVersion> keys) {
        setKeys(keys);
        return this;
    }

    public List<KeyVersion> getKeys() {
        return keys;
    }


    public DeleteObjectsRequest withKeys(String... keys) {
        List<KeyVersion> keyVersions = new ArrayList<KeyVersion>(keys.length);
        for (String key : keys) {
            keyVersions.add(new KeyVersion(key));
        }
        setKeys(keyVersions);
        return this;
    }

    @Override
    public void validate() {
        if(this.keys.isEmpty()) throw new NullPointerException(getErrorMessage("key"));
    }

    public static class KeyVersion implements Serializable {

        private static final long serialVersionUID = -805573587867125275L;
        private final String key;
        private final String version;

        /**
         * Constructs a key without a version.
         */
        public KeyVersion(String key) {
            this(key, null);
        }

        /**
         * Constructs a key-version pair.
         */
        public KeyVersion(String key, String version) {
            this.key = key;
            this.version = version;
        }

        public String getKey() {
            return key;
        }

        public String getVersion() {
            return version;
        }
    }
}
