package com.yovole.sdk.oss.model;

import com.yovole.sdk.oss.exception.SdkClientException;

import java.util.Collections;
import java.util.Date;
import java.util.Map;
import java.util.TreeMap;

public class ObjectMetadata extends BaseModel
{
    private static final long serialVersionUID = 2384266828885489491L;

    private Map<String, String> userMetadata = new TreeMap<String, String>(String.CASE_INSENSITIVE_ORDER);

    private Map<String, Object> metadata = new TreeMap<String, Object>(String.CASE_INSENSITIVE_ORDER);

    public static final String AES_256_SERVER_SIDE_ENCRYPTION =
            SSEAlgorithm.AES256.getAlgorithm();

    private Date httpExpiresDate;

    private Date expirationTime;

    private String expirationTimeRuleId;

    private Boolean ongoingRestore;

    private Date restoreExpirationTime;

    public ObjectMetadata() {}

    private ObjectMetadata(ObjectMetadata from) {
        this.userMetadata = from.userMetadata == null
            ? null
            : new TreeMap<String,String>(from.userMetadata);
        // shallow clone the meata data
        this.metadata = from.metadata == null
            ? null
            : new TreeMap<String, Object>(from.metadata);
        this.expirationTime = cloneDate(from.expirationTime);
        this.expirationTimeRuleId = from.expirationTimeRuleId;
        this.httpExpiresDate = cloneDate(from.httpExpiresDate);
        this.ongoingRestore = from.ongoingRestore;
        this.restoreExpirationTime = cloneDate(from.restoreExpirationTime);
    }

    public static Date cloneDate(Date date) {
        return date == null ? null : new Date(date.getTime());
    }

    public Map<String, String> getUserMetadata() {
        return userMetadata;
    }

    public void setUserMetadata(Map<String, String> userMetadata) {
        this.userMetadata = userMetadata;
    }

    public void setHeader(String key, Object value) {
        metadata.put(key, value);
    }

    public void addUserMetadata(String key, String value) {
        this.userMetadata.put(key, value);
    }

    public Map<String, Object> getRawMetadata() {
        Map<String,Object> copy = new TreeMap<String,Object>(String.CASE_INSENSITIVE_ORDER);
        copy.putAll(metadata);
        return Collections.unmodifiableMap(copy);
    }

    public Object getRawMetadataValue(String key) {
        return metadata.get(key);
    }

    public void setRawMetadata(Map<String, Object> metadata){
        this.metadata = metadata;
    }

    public Date getLastModified() {
        return cloneDate((Date)metadata.get(Headers.LAST_MODIFIED));
    }


    public void setLastModified(Date lastModified) {
        metadata.put(Headers.LAST_MODIFIED, lastModified);
    }

    public long getContentLength() {
        Long contentLength = (Long)metadata.get(Headers.CONTENT_LENGTH);

        if (contentLength == null) return 0;
        return contentLength.longValue();
    }

    public long getInstanceLength() {

        String contentRange = (String)metadata.get(Headers.CONTENT_RANGE);
        if (contentRange != null) {
            int pos = contentRange.lastIndexOf("/");
            if (pos >= 0)
                return Long.parseLong(contentRange.substring(pos+1));
        }
        return getContentLength();
    }

    public void setContentLength(long contentLength) {
        metadata.put(Headers.CONTENT_LENGTH, contentLength);
    }

    public String getContentType() {
        return (String)metadata.get(Headers.CONTENT_TYPE);
    }

    public void setContentType(String contentType) {
        metadata.put(Headers.CONTENT_TYPE, contentType);
    }

    public String getContentLanguage() {
        return (String)metadata.get(Headers.CONTENT_LANGUAGE);
    }

    public void setContentLanguage(String contentLanguage) {
        metadata.put(Headers.CONTENT_LANGUAGE, contentLanguage);
    }

    public String getContentEncoding() {
        return (String)metadata.get(Headers.CONTENT_ENCODING);
    }

    public void setContentEncoding(String encoding) {
        metadata.put(Headers.CONTENT_ENCODING, encoding);
    }

    public String getCacheControl() {
        return (String)metadata.get(Headers.CACHE_CONTROL);
    }

    public void setCacheControl(String cacheControl) {
        metadata.put(Headers.CACHE_CONTROL, cacheControl);
    }

    public void setContentMD5(String md5Base64) {
        if(md5Base64 == null){
            metadata.remove(Headers.CONTENT_MD5);
        }else{
            metadata.put(Headers.CONTENT_MD5, md5Base64);
        }

    }

    public String getContentMD5() {
        return (String)metadata.get(Headers.CONTENT_MD5);
    }

    public void setContentDisposition(String disposition) {
        metadata.put(Headers.CONTENT_DISPOSITION, disposition);
    }

    public String getContentDisposition() {
        return (String)metadata.get(Headers.CONTENT_DISPOSITION);
    }

    public String getETag() {
        return (String)metadata.get(Headers.ETAG);
    }

    public String getVersionId() {
        return (String)metadata.get(Headers.S3_VERSION_ID);
    }

    public String getSSEAlgorithm() {
        return (String)metadata.get(Headers.SERVER_SIDE_ENCRYPTION);
    }

    @Deprecated
    public String getServerSideEncryption() {
        return (String)metadata.get(Headers.SERVER_SIDE_ENCRYPTION);
    }

    public void setSSEAlgorithm(String algorithm) {
        metadata.put(Headers.SERVER_SIDE_ENCRYPTION, algorithm);
    }

    @Deprecated
    public void setServerSideEncryption(String algorithm) {
        metadata.put(Headers.SERVER_SIDE_ENCRYPTION, algorithm);
    }

    public String getSSECustomerAlgorithm() {
        return (String) metadata.get(Headers.SERVER_SIDE_ENCRYPTION_CUSTOMER_ALGORITHM);
    }

    public void setSSECustomerAlgorithm(String algorithm) {
        metadata.put(Headers.SERVER_SIDE_ENCRYPTION_CUSTOMER_ALGORITHM, algorithm);
    }

    public String getSSECustomerKeyMd5() {
        return (String)metadata.get(Headers.SERVER_SIDE_ENCRYPTION_CUSTOMER_KEY_MD5);
    }

    /**
     * For internal use only. This method is only used to set the value in the
     * object after receiving the value in a response from S3. When sending
     * requests, use  members in request objects.
     */
    public void setSSECustomerKeyMd5(String md5Digest) {
        metadata.put(Headers.SERVER_SIDE_ENCRYPTION_CUSTOMER_KEY_MD5, md5Digest);
    }

    /**
     * Returns the time this object will expire and be completely removed from
     * S3. Returns null if this object will never expire.
     */
    public Date getExpirationTime() {
        return cloneDate(expirationTime);
    }

    /**
     * For internal use only. This will *not* set the object's expiration time,
     * and is only used to set the value in the object after receiving the value
     * in a response from S3.
     *
     * @param expirationTime
     *            The expiration time for the object.
     */
    public void setExpirationTime(Date expirationTime) {
        this.expirationTime = expirationTime;
    }

    /**
     * Returns the rule ID for this
     * object's expiration, or null if it doesn't expire.
     */
    public String getExpirationTimeRuleId() {
        return expirationTimeRuleId;
    }

    /**
     * For internal use only. This will *not* set the object's expiration time
     * rule id, and is only used to set the value in the object after receiving
     * the value in a response from S3.
     */
    public void setExpirationTimeRuleId(String expirationTimeRuleId) {
        this.expirationTimeRuleId = expirationTimeRuleId;
    }


    /**
     * Returns the time at which an object that has been temporarily restored
     * from Amazon Glacier will expire, and will need to be restored again in
     * order to be accessed. Returns null if this is not a temporary copy of an
     * object restored from Glacier.
     */
    public Date getRestoreExpirationTime() {
        return cloneDate(restoreExpirationTime);
    }

    /**
     * For internal use only. This will *not* set the object's restore
     * expiration time, and is only used to set the value in the object after
     * receiving the value in a response from S3.
     *
     * @param restoreExpirationTime
     *            The new restore expiration time for the object.
     */
    public void setRestoreExpirationTime(Date restoreExpirationTime) {
        this.restoreExpirationTime = restoreExpirationTime;
    }

    /**
     * For internal use only. Sets the boolean value which indicates whether
     * there is ongoing restore request. Not intended to be called by external
     * code.
     */
    public void setOngoingRestore(boolean ongoingRestore) {
        this.ongoingRestore = Boolean.valueOf(ongoingRestore);
    }


    /**
     *  Returns the boolean value which indicates whether there is ongoing restore request.
     */
    public Boolean getOngoingRestore() {
        return this.ongoingRestore;
    }

    /**
     *  Set the date when the object is no longer cacheable.
     */
    public void setHttpExpiresDate(Date httpExpiresDate) {
        this.httpExpiresDate = httpExpiresDate;
    }

    /**
     * Returns the date when the object is no longer cacheable.
     */
    public Date getHttpExpiresDate() {
        return cloneDate(httpExpiresDate);
    }

    /**
     * @return The storage class of the object. Returns null if the object is in STANDARD storage.
     *         See   for possible values
     */
    public String getStorageClass() {
        final Object storageClass = metadata.get(Headers.STORAGE_CLASS);
        if (storageClass == null) {
            return null;
        }
        return storageClass.toString();
    }

    /**
     * Returns the value of the specified user meta datum.
     */
    public String getUserMetaDataOf(String key) {
        return userMetadata == null ? null : userMetadata.get(key);
    }

    public ObjectMetadata clone() {
        return new ObjectMetadata(this);
    }

    public String getSSEAwsKmsKeyId() {
        return (String) metadata
                .get(Headers.SERVER_SIDE_ENCRYPTION_AWS_KMS_KEYID);
    }

    public Integer getPartCount() {
        return (Integer) metadata.get(Headers.S3_PARTS_COUNT);
    }

    public Long[] getContentRange() {
        String contentRange = (String) metadata.get(Headers.CONTENT_RANGE);
        Long[] range = null;
        if (contentRange != null) {
            String[] tokens = contentRange.split("[ -/]+");
            try {
                range = new Long[] { Long.parseLong(tokens[1]), Long.parseLong(tokens[2]) };
            } catch (NumberFormatException nfe) {
                throw new SdkClientException(
                        "Unable to parse content range. Header 'Content-Range' has corrupted data" + nfe.getMessage(),
                        nfe);
            }
        }
        return range;
    }

    public String getReplicationStatus() {
        return (String) metadata.get(Headers.OBJECT_REPLICATION_STATUS);
    }
}
