package com.yovole.sdk.oss.model;

import java.io.File;
import java.io.InputStream;
import java.util.Objects;

public class PutObjectRequest extends BaseRequest implements S3DataSource{
    private static final long serialVersionUID = -5395579163006513352L;
    private File file;
    private String redirectLocation;
    private transient InputStream inputStream;
    private ObjectMetadata metadata;
    private CannedAccessControlList cannedAcl;

    public PutObjectRequest(String bucketName, String key, File file) {
        this.setBucket(bucketName);
        this.setKey(key);
        this.file = file;
    }

//    public PutObjectRequest(String bucketName, String key, String redirectLocation) {
//        this.bucketName = bucketName;
//        this.key = key;
//        this.redirectLocation = redirectLocation;
//    }

    public PutObjectRequest(String bucketName, String key, InputStream input,
                            ObjectMetadata metadata) {
        this.setBucket(bucketName);
        this.setKey(key);
        this.inputStream = input;
        this.metadata = metadata;
    }
    @Override
    public File getFile() {
        return file;
    }
    @Override
    public void setFile(File file) {
        this.file = file;
    }

    public String getRedirectLocation() {
        return redirectLocation;
    }

    public void setRedirectLocation(String redirectLocation) {
        this.redirectLocation = redirectLocation;
    }
    @Override
    public InputStream getInputStream() {
        return inputStream;
    }
    @Override
    public void setInputStream(InputStream inputStream) {
        this.inputStream = inputStream;
    }

    public ObjectMetadata getMetadata() {
        return metadata;
    }

    public void setMetadata(ObjectMetadata metadata) {
        this.metadata = metadata;
    }

    @Override
    public void validate() {
        Objects.requireNonNull(this.getBucket(),getErrorMessage("bucketName"));
        Objects.requireNonNull(this.getKey(),getErrorMessage("key"));
        if (this.inputStream == null && this.file == null) {
            throw new NullPointerException(getErrorMessage("inputStream or file"));
        }
    }

    public CannedAccessControlList getCannedAcl() {
        return cannedAcl;
    }

    public void setCannedAcl(CannedAccessControlList cannedAcl) {
        this.cannedAcl = cannedAcl;
    }
}
