package com.yovole.sdk.oss.utils;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.PropertyNamingStrategy;
import org.apache.log4j.Logger;

import java.util.List;
import java.util.Map;

public class JacksonUtils {
    public static Map<String, Object> objectToMap(Object obj, PropertyNamingStrategy strategy) {
        return getMapper(strategy).convertValue(obj, new TypeReference<Map<String, Object>>() {
        });
    }

    public static <T> T jsonToObj(String json, TypeReference valueTypeRef, PropertyNamingStrategy strategy) {
        try {
            return getMapper(strategy).readValue(json, valueTypeRef);
        } catch (Exception e) {
            Logger.getLogger(JacksonUtils.class).error(e.getMessage());
            return null;
        }
    }

    public static <T> T jsonToObj(String json, Class<T> tClass, PropertyNamingStrategy strategy) {
        try {
            return getMapper(strategy).readValue(json, tClass);
        } catch (Exception e) {
            Logger.getLogger(JacksonUtils.class).error(e.getMessage());
            return null;
        }
    }

    public <T> T jsonToObj(String json, TypeReference valueTypeRef) {
        return jsonToObj(json, valueTypeRef, null);
    }

    public static <T> T jsonToObj(String json, Class<T> tClass) {
        return jsonToObj(json, tClass, null);
    }

    public static String objToString(Object obj) {
        return objToString(obj, null);
    }

    public static String objToString(Object obj, PropertyNamingStrategy strategy) {
        try {
            return getMapper(strategy).writeValueAsString(obj);
        } catch (JsonProcessingException e) {
            Logger.getLogger(JacksonUtils.class).error(e.getMessage());
            return null;
        }
    }

    public static String getQueryStr(Object obj){
        Map<String, Object> map = getMapper(null).convertValue(obj, new TypeReference<Map<String, Object>>() {
        });
        StringBuilder stringBuilder = new StringBuilder();
        for(String key : map.keySet()){
            if(map.get(key) instanceof List) {
                List<Object> list = (List<Object>) map.get(key);
                for(Object object : list){
                    stringBuilder.append("&").append(key).append("=").append(object);
                }
            } else {
                stringBuilder.append("&").append(key).append("=").append(map.get(key));
            }
        }
        return stringBuilder.toString();
    }

    private static ObjectMapper getMapper(PropertyNamingStrategy strategy){
        ObjectMapper oMapper = new ObjectMapper();
        oMapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);
        oMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        if (strategy != null) oMapper.setPropertyNamingStrategy(strategy);
        return oMapper;
    }

}
