package com.yovole.sdk.oss.utils;

import com.yovole.sdk.oss.exception.SdkClientException;
import com.yovole.sdk.oss.model.Headers;
import com.yovole.sdk.oss.model.ObjectMetadata;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.log4j.Logger;

import java.text.ParseException;
import java.util.HashSet;
import java.util.Set;

public class MetadataUtils {
    private static Logger logger = Logger.getLogger(MetadataUtils.class);
    private static final Set<String> ignoredHeaders = new HashSet();

    public static ObjectMetadata populateObjectMetadata(HttpResponse response, ObjectMetadata metadata) {
        for(Header header : response.getAllHeaders()){
            String key = header.getName();
            if (key.startsWith(Headers.S3_USER_METADATA_PREFIX)) {
                key = key.substring(Headers.S3_USER_METADATA_PREFIX.length());
                metadata.addUserMetadata(key, header.getValue());
            } else if (!ignoredHeaders.contains(key)) {
                if (key.equalsIgnoreCase("Last-Modified")) {
                    try {
                        metadata.setHeader(key,DateUtils.parseRFC822Date(header.getValue()));
                    } catch (ParseException e) {
                        logger.error("parse Last-Modified date error,str:" + header.getValue());
                    }
                } else if (key.equalsIgnoreCase("Content-Length")) {
                    try {
                        metadata.setHeader(key, Long.parseLong(header.getValue()));
                    } catch (NumberFormatException nfe) {
                        throw new SdkClientException("Unable to parse content length. Header 'Content-Length' has corrupted data" + nfe.getMessage(), nfe);
                    }
                } else if (key.equalsIgnoreCase("ETag")) {
                    metadata.setHeader(key, header.getValue().replaceAll("\"", ""));
                } else if (key.equalsIgnoreCase("Expires")) {
                    metadata.setHeader("Expires", header.getValue());
                    if(!"0".equals(header.getValue())) {
                        try {
                            metadata.setHttpExpiresDate(DateUtils.parseRFC822Date(header.getValue()));
                        } catch (Exception pe) {
                            logger.warn("Unable to parse http expiration date: " + header.getValue());
                        }
                    }
                } else if (key.equalsIgnoreCase("x-amz-mp-parts-count")) {
                    try {
                        metadata.setHeader(key, Integer.parseInt((String)header.getValue()));
                    } catch (NumberFormatException var7) {
                        throw new SdkClientException("Unable to parse part count. Header x-amz-mp-parts-count has corrupted data" + var7.getMessage(), var7);
                    }
                } else {
                    metadata.setHeader(key, header.getValue());
                }
            }
        }
        return metadata;
    }

    static {
        ignoredHeaders.add("Date");
        ignoredHeaders.add("Server");
        ignoredHeaders.add("x-amz-request-id");
        ignoredHeaders.add("x-amz-id-2");
        ignoredHeaders.add("X-Amz-Cf-Id");
        ignoredHeaders.add("Connection");
    }
}
