package com.yovole.sdk.oss;

import com.yovole.sdk.oss.model.*;
import org.apache.http.entity.ContentType;
import org.junit.Before;
import org.junit.Test;
import java.io.File;
import java.io.FileInputStream;
import java.util.*;
import org.apache.log4j.Logger;

public class OssApiTest {
    //protected Logger logger = LoggerFactory.getLogger(getClass());
    protected Logger logger = Logger.getLogger(getClass());
    private IOssApi ossApi;

    @Before
    public void setUp() throws Exception {
        ossApi = new OssApi("http://cn-east-1.cloud-oss.com","d1a190cf-b05e-4ca4-9a24-3c3ac7938e08", "24ecc729-92e5-45bc-a534-6d491839da13");
        //ossApi = new OssApi("http://cn-east-1.cloud-oss.com","f570b245-490b-48f0-bfb6-6dce036632bf");
    }
    @Test
    public void createBucket() {
        CreateBucketRequest request = new CreateBucketRequest();
        request.setProjectId("8a83a00067dfa0ba0167dffc4683005b");
        request.setBucket("lj-test-123");
        request.setCannedAcl(CannedAccessControlList.PublicRead);//x-amz-acl
        request.withUserInfo("VPP-ID","8a83a0ff67dfa0bc0167dfa77cad0000","USER-ID");
        logger.debug(ossApi.createBucket(request));
    }

    @Test
    public void listBucket() {
        ListBucketRequest request = new ListBucketRequest();
        //request.withUserInfo("vppId","customerId","userId");
        request.setProjectId("2c9481226684eb130166854b595e0035");
        //request.setName("test2");
        logger.debug(ossApi.listBucket(request));
    }

    @Test
    public void deleteBucket() {
        DeleteBucketRequest request = new DeleteBucketRequest();
        request.setBucket("lj-test-006");
        request.setPurgeObjects(true);
        ossApi.deleteBucket(request);
    }

    @Test
    public void setBucketAcl() {
        SetBucketAclRequest request = new SetBucketAclRequest("lj-test-004",CannedAccessControlList.PublicRead);
        logger.debug(ossApi.setBucketAcl(request));
    }

    @Test
    public void listObject() {
        ListObjectRequest request = new ListObjectRequest();
        request.setMaxKeys(5);
        request.setBucket("lj-test-002");//vpf"test2","4.txt"ileupload/5.txt
        //request.setPrefix("vp/");
        request.setDelimiter("/");
        //request.setPrefix("iTool");lj-test-002
        logger.debug(ossApi.listObject(request));
    }


    @Test
    public void setObjectAcl() {
        SetObjectAclRequest request = new SetObjectAclRequest("lj-test-002","1.txt",CannedAccessControlList.Private);
        ossApi.setObjectAcl(request);
    }

    @Test
    public void getObjectAcl() {
        logger.debug(ossApi.getObjectAcl(new ObjectInfoRequest("lj-test-002","1.txt")));
    }

    @Test
    public void getObjectInfo() {
        logger.debug(ossApi.getObjectInfo(new ObjectInfoRequest("lj-test-002","1.txt")));
    }

    @Test
    public void getObjectMetadata() {
        //logger.debug("{}",ossApi.getObjectMetadata(new ObjectInfoRequest("testpolicy1","1.txt")));
    }

    @Test
    public void deleteObject() {
        ossApi.deleteObject(new ObjectInfoRequest("lj-test-002","1.txt"));
//        ObjectDeleteRequest request = new ObjectDeleteRequest();
//        request.setBucketName("test2");
//        request.setObject("root-8.txt");
//        ossApi.deleteObject(request);
    }

    @Test
    public void putObject() throws Exception{
        File file = new File("/Users/Sophie/Downloads/blob_test.mp4");
        PutObjectRequest request = new PutObjectRequest("lj-test-002","3.mp4",file);
        request.setCannedAcl(CannedAccessControlList.PublicRead);
        PutObjectResult result = ossApi.putObject(request);
    }

    @Test
    public void putObject1() throws Exception{
        FileInputStream fileInputStream = new FileInputStream("/Users/Sophie/Downloads/blob_test.mp4");
//        ByteArrayInputStream input = new ByteArrayInputStream("Hello World!5".getBytes());
        ObjectMetadata metadata = new ObjectMetadata();
        Map<String,String> mapdata = new HashMap<>();
        mapdata.put("sdk","4");
        metadata.setUserMetadata(mapdata);
        metadata.setContentType(ContentType.APPLICATION_OCTET_STREAM.getMimeType());
        //logger.debug(input.available());
        //metadata.setContentLength(input.available());

//        14:16:03.834 [main] DEBUG org.apache.http.headers - http-outgoing-0 >> Content-Length: 254
//        14:16:03.834 [main] DEBUG org.apache.http.headers - http-outgoing-0 >> Connection: Keep-Alive
//        14:16:03.834 [main] DEBUG org.apache.http.headers - http-outgoing-0 >> Expect: 100-continue
        PutObjectRequest request = new PutObjectRequest("lj-test-002","blob_test5555.mp4",fileInputStream,metadata);
        request.setCannedAcl(CannedAccessControlList.PublicRead);
        logger.debug(ossApi.putObject(request));
    }

    @Test
    public void deleteObjects() throws Exception{
        DeleteObjectsRequest request = new DeleteObjectsRequest("testpolicy1");
        request.withKeys(new String[]{"1.txt","2.txt"});
        DeleteObjectsResult result = ossApi.deleteObjects(request);
        //logger.debug("{}",result);
    }


    @Test
    public void copyObject() {
        CopyObjectRequest request = new CopyObjectRequest("lj-test-002","1.txt","lj-test-002","2.txt");
        ObjectMetadata metadata = new ObjectMetadata();
        Map<String,String> mapdata = new HashMap<>();
        mapdata.put("aaaa","222");
        metadata.setUserMetadata(mapdata);
        request.setMetadataDirective(MetadataDirective.COPY.name());
        request.setNewObjectMetadata(metadata);
        logger.debug(ossApi.copyObject(request));

    }

    @Test
    public void copyObject1(){
        CopyObjectRequest request = new CopyObjectRequest("lj-test-002", "1.txt","lj-test-002", "2.txt");
        //request.withMetadataDirective(MetadataDirective.COPY);
        ObjectMetadata metadata = new ObjectMetadata();
        //metadata.setContentType("text/plain");
        metadata.setContentEncoding("gzip,deflate,compress");
        metadata.setContentDisposition("attachment; filename = 1.txt");
        metadata.setCacheControl("no-cache, no-store, max-age=0, must-revalidate");
        metadata.setContentLanguage("en");
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DATE,1);
        metadata.setHttpExpiresDate(calendar.getTime());
        metadata.getUserMetadata().put("lj","9704");
        request.setNewObjectMetadata(metadata);
        logger.debug(ossApi.copyObject(request));
    }

    @Test
    public void getObject() throws Exception{
        S3Object s3Object = ossApi.getObject(
                new GetObjectRequest("lj-test-002", "upload_program.txt"));
        byte buffer[]=new byte[1024];
        StringBuilder sb=new StringBuilder();
        while(s3Object.getObjectContent().read(buffer,0,1024)!=-1){
            sb.append(new String(buffer));
            logger.debug(new String(buffer));
        }
    }

    @Test
    public void getObject1() throws Exception{
        S3Object s3Object = ossApi.getObject(
                new GetObjectRequest("lj-test-002", "1543980335023.jpg"));
        logger.debug("get object end");
    }

    @Test
    public void getObject2() throws Exception{
        File file = new File("/Users/Sophie/Downloads/Spring Boot 2.0 Web Applications-1.mp4");
        ObjectMetadata metadata = ossApi.getObject(new GetObjectRequest("lj-test-002", "Spring Boot 2.0 Web Applications.mp4"),file);
        logger.debug(metadata);
    }

    @Test
    public void createDir() {
        ossApi.createDir(new ObjectInfoRequest("testpolicy1","test1/"));
    }

    @Test
    public void setObjectMainMetadata() {
        ObjectMetadata metadata = new ObjectMetadata();
//        metadata.setContentType("text/plain");
//        metadata.setContentEncoding("gzip,deflate,compress");
//        metadata.setContentDisposition("attachment; filename = 1.txt");
//        metadata.setCacheControl("no-cache, no-store, max-age=0, must-revalidate");
//        metadata.setContentLanguage("en");
//        metadata.setContentType("11");
//        metadata.setContentEncoding("22");
//        metadata.setContentDisposition("33");
//        metadata.setCacheControl("44");
//        metadata.setContentLanguage("55");
//        metadata.setHeader("Expires","66");
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DATE,1);
        metadata.setHttpExpiresDate(calendar.getTime());
        SetObjectMainMetadataRequest request = new SetObjectMainMetadataRequest("lj-test-002", "1.txt",metadata);
        logger.debug(ossApi.setObjectMainMetadata(request));
    }

    @Test
    public void setObjectUserMetadata() {
        SetObjectUserMetadataRequest request = new SetObjectUserMetadataRequest("testpolicy1", "1.txt");
        request.getUserMetadata().put("lj1","9704");
        //logger.debug("{}",ossApi.setObjectUserMetadata(request));
    }

    @Test
    public void setObjectMetadata() {
        ObjectMetadata metadata = new ObjectMetadata();
//        metadata.setContentType("text/plain");
//        metadata.setContentEncoding("gzip,deflate,compress");
//        metadata.setContentDisposition("attachment; filename = 1.txt");
//        metadata.setCacheControl("no-cache, no-store, max-age=0, must-revalidate");
//        metadata.setContentLanguage("en");

//        metadata.setContentType("11");
//        metadata.setContentEncoding("22");
//        metadata.setContentDisposition("33");
//        metadata.setCacheControl("44");
//        metadata.setContentLanguage("55");
//        metadata.setHeader("Expires","3333");
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DATE,1);
        metadata.setHttpExpiresDate(calendar.getTime());
        metadata.getUserMetadata().put("lj","9704");
        SetObjectMetadataRequest request = new SetObjectMetadataRequest("test5", "00map.html",metadata);
        //logger.debug("{}",ossApi.setObjectMetadata(request));
    }

    @Test
    public void test(){
//        PutObjectRequest request = new PutObjectRequest("a","b",new File("a"));
//        if(request instanceof S3DataSource){
//            logger.debug("111");
//        }



    }


    @Test
    public void uploadBigObject() {
        UploadBigFileRequest request = new UploadBigFileRequest();
        File file = new File("/Users/Sophie/Downloads/1534846566426.jpg");
        request.setFile(file);
        //ossApi.uploadBigObject(request);
    }


    @Test
    public void initiateMultipartUpload() {//YF-有孚云帮助中心-03存储-对象存储.docx
        InitiateMultipartUploadRequest request = new InitiateMultipartUploadRequest("lj-test-002","QQ_V6.5.0.dmg");
        request.setCannedACL(CannedAccessControlList.PublicRead);
        logger.debug(ossApi.initiateMultipartUpload(request));
    }

    @Test
    public void listMultipartUploads() {
        ListMultipartUploadsRequest request = new ListMultipartUploadsRequest("lj-test-002");
        request.setMaxUploads(5);
        //request.setKeyMarker("1534846566426.jpg");
        //request.setUploadIdMarker("2~nxfvQVuLAFI7yAKEUHXd0XwYCjE0Qkn");
//        request.setPrefix("ljtest1");
        request.setDelimiter("/");
        logger.debug(ossApi.listMultipartUploads(request));
    }

    @Test
    public void doRequest() {
    }

    @Test
    public void doRequest1() {
    }

    @Test
    public void doHttpRequest() {
    }

    @Test
    public void getHttpClient() {
    }

    @Test
    public void doRequestWithoutResponse() {
        List<PartETag> partETags = new ArrayList<>();
        PartETag partETag = new PartETag(1,"werwerewrwer");
        partETags.add(partETag);
        CompleteMultipartUploadRequest request1 = new CompleteMultipartUploadRequest("testpolicy1","QQ_V6.5.0.dmg","tttt",partETags);
      //  logger.debug(JacksonUtils.objToString(request1));
    }

    @Test
    public void listParts() {
        ListPartsRequest request = new ListPartsRequest("lj-test-002","QQ_V6.5.0.dmg","2~fDm836prWZZ7bUeHIbf0itmBSCpU61f");
        request.setMaxParts(10);
        logger.debug(ossApi.listParts(request));
    }


    @Test
    public void uploadPart() {
        long PART_SIZE = 10*1024*1024;
        File file = new File("/Users/Sophie/Downloads/QQ_V6.5.0.dmg");
        String bucket = "lj-test-002";
        String key = "QQ_V6.5.0.dmg";
        String uploadId = "2~fDm836prWZZ7bUeHIbf0itmBSCpU61f";
        List<PartETag> partETags = new ArrayList<>();
        long contentLength = file.length();
        long partSize;
        try {
            long compSize = 0;
            for (int i = 1; compSize < contentLength; i++) {
                partSize = Math.min(PART_SIZE, (contentLength - compSize));
                UploadPartRequest uploadRequest = new UploadPartRequest()
                        .withBucketName(bucket)
                        .withKey(key)
                        .withUploadId(uploadId).withPartNumber(i)
                        .withFileOffset(compSize).withFile(file).withPartSize(partSize);
                if(compSize + partSize >= contentLength) uploadRequest.setLastPart(true);
                partETags.add(ossApi.uploadPart(uploadRequest).getPartETag());
                compSize += partSize;
                if(i==2) break;//只上传2块
            }
        } catch (Exception e) {
            //   logger.error(e.getMessage(), e);
            ossApi.abortMultipartUpload(new AbortMultipartUploadRequest(bucket, key, uploadId));
        }
    }

    @Test
    public void uploadPart1() {
        long PART_SIZE = 10*1024*1024;
        File file = new File("/Users/Sophie/Downloads/1.mp4");
        String bucket = "lj-test-002";
        String key = "5.mp4";
        InitiateMultipartUploadRequest request = new InitiateMultipartUploadRequest(bucket,key);
        String uploadId = ossApi.initiateMultipartUpload(request).getUploadId();
        List<PartETag> partETags = new ArrayList<>();
        long contentLength = file.length();
        long partSize;
        try {
            long compSize = 0;
            for (int i = 1; compSize < contentLength; i++) {
                partSize = Math.min(PART_SIZE, (contentLength - compSize));
                UploadPartRequest uploadRequest = new UploadPartRequest()
                        .withBucketName(bucket)
                        .withKey(key)
                        .withUploadId(uploadId).withPartNumber(i)
                        .withFileOffset(compSize).withFile(file).withPartSize(partSize);
                if(compSize + partSize >= contentLength) uploadRequest.setLastPart(true);
                partETags.add(ossApi.uploadPart(uploadRequest).getPartETag());
                compSize += partSize;
            }
            CompleteMultipartUploadRequest request1 = new CompleteMultipartUploadRequest(bucket,key,uploadId,partETags);
            ossApi.completeMultiPartUpload(request1);
        } catch (Exception e) {
         //   logger.error(e.getMessage(), e);
            ossApi.abortMultipartUpload(new AbortMultipartUploadRequest(bucket, key, uploadId));
        }
    }


    @Test
    public void completeMultiPartUpload() {
        ListPartsRequest request = new ListPartsRequest("testpolicy1","QQ_V6.5.0.dmg","2~whZ8L8XsLmZjRiBgcWkpXbXLHumLh12");
        PartListing partListing = ossApi.listParts(request);
        List<PartETag> partETags = new ArrayList<>();
        for(PartSummary summary : partListing.getParts()){
            partETags.add(new PartETag(summary.getPartNumber(),summary.getETag()));
        }
        CompleteMultipartUploadRequest request1 = new CompleteMultipartUploadRequest("testpolicy1","QQ_V6.5.0.dmg","2~whZ8L8XsLmZjRiBgcWkpXbXLHumLh12",partETags);
       // logger.debug("{}",ossApi.completeMultiPartUpload(request1));
    }

    @Test
    public void abortMultipartUpload() {
        AbortMultipartUploadRequest request = new AbortMultipartUploadRequest("lj-test-002","QQ_V6.5.0.dmg","2~fDm836prWZZ7bUeHIbf0itmBSCpU61f");
        ossApi.abortMultipartUpload(request);
    }
}